/* -*- Mode: C; c-file-style: "gnu" -*-
   jnirefl.c -- Java Native Interface methods relating reflection
   Created: Hernan Otero <hernan_otero@bigfoot.com>, 10-Jul-1998
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "config.h"

#include "jni.h"
#include "jniint.h"
#include "nativeglue.h"

/**
 * XXX Waiting for JDK 1.2 JNI compatibility
 * Should be a JNI entry
 */
jmethodID
FromReflectedMethod(JNIEnv *env, jobject method)
{
  /* Since a Method is only valid for a single Class and its
   * ancestors and descendants which have that method (and I'm
   * not even sure about the ancestors part), the jmethodID
   * returned will always be the same (jmethodIDs are likewise
   * valid for ancestors and descendants of a single Class).
   * Thus it is safe to cache the jmethodID.
   * Same argument is valid for Field/jfieldID.
   */
  return (jmethodID) NSA_GetNativeState(method);
}

/**
 * XXX Waiting for JDK 1.2 JNI compatibility
 * Should be a JNI entry
 */
jfieldID
FromReflectedField(JNIEnv *env, jobject field)
{
  return (jfieldID) NSA_GetNativeState(field);
}

/**
 * XXX Waiting for JDK 1.2 JNI compatibility
 * Should be a JNI entry
 */
jobject
ToReflectedMethod(JNIEnv *env, jmethodID method)
{
  jclass class_class = (*env)->FindClass(env, "java/lang/Class");
  jboolean is_ctor = (strcmp(method->name, "<init>") == 0);
  jclass method_class = (*env)->FindClass(env,
      is_ctor ? "java/lang/reflect/Constructor"
	  : "java/lang/reflect/Method");
  jfieldID clazz_field = (*env)->GetFieldID(env, method_class,
      "clazz", "Ljava/lang/Class;");
  jfieldID slot_field = (*env)->GetFieldID(env, method_class,
      "slot", "I");
  jfieldID name_field = is_ctor ? NULL
      : (*env)->GetFieldID(env, method_class,
	  "name", "Ljava/lang/String;");
  jfieldID returnType_field = is_ctor ? NULL
      : (*env)->GetFieldID(env, method_class,
      "returnType", "Ljava/lang/Class;");
  jfieldID parameterTypes_field = (*env)->GetFieldID(env, method_class,
      "parameterTypes", "[Ljava/lang/Class;");
  jfieldID exceptionTypes_field = (*env)->GetFieldID(env, method_class,
      "exceptionTypes", "[Ljava/lang/Class;");
  jmethodID method_ctor = (*env)->GetMethodID(env, method_class,
      "<init>", "()V");
  return to_reflected_method(env, method, class_class, method_class,
      method_ctor, clazz_field, slot_field, name_field,
      returnType_field, parameterTypes_field, exceptionTypes_field);
}

/**
 * XXX Waiting for JDK 1.2 JNI compatibility
 * Should be a JNI entry
 */
jobject
ToReflectedField(JNIEnv *env, jfieldID field)
{
  jclass field_class = (*env)->FindClass(env,
	    "java/lang/reflect/Field");
  jfieldID clazz_field = (*env)->GetFieldID(env, field_class,
	    "clazz", "Ljava/lang/Class;");
  jfieldID slot_field = (*env)->GetFieldID(env, field_class,
	    "slot", "I");
  jfieldID name_field = (*env)->GetFieldID(env, field_class,
	    "name", "Ljava/lang/String;");
  jfieldID type_field = (*env)->GetFieldID(env, field_class,
	    "type", "Ljava/lang/Class;");
  jmethodID field_ctor = (*env)->GetMethodID(env, field_class,
	    "<init>", "()V");
  return to_reflected_field(env, field, field_class, field_ctor,
      slot_field, type_field, name_field, clazz_field);
}
