/* -*- Mode: C; c-file-style: "gnu" -*-
   component.c -- native methods for sun/awt/motif/MComponentPeer
   Created: Chris Toshok <toshok@hungry.com>, 3-Mar-1998.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#include "config.h"
#include "jni.h"
#include "log.h"
#include "common.h"

#include <assert.h>

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pInitialize(JNIEnv *env,
					      jobject peer)
{
  Widget w = get_component_widget(env, peer);
  JAVARLOG1 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_pInitialize(peer = %p)\n", peer);

  if (NULL == w)
    {
      JAVARLOG0(AWT_LOG, 1, "widget is NULL in MComponentPeer.pInitialize!\n");
      return;
    }

  add_component_event_handler(w, peer);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pShow(JNIEnv *env,
					jobject peer)
{
  Widget w = get_component_widget(env, peer);

  JAVARLOG1 (AWT_LOG, 2, "in sun_awt_motif_MComponentPeer_pShow(peer = %p)\n", peer);

  if (NULL == w)
    {
      JAVARLOG0(AWT_LOG, 1, "widget is NULL in MComponentPeer.pShow!\n");
      return;
    }

  LOCK_AWT();
  XtManageChild(w);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pHide(JNIEnv *env,
					jobject peer)
{
  Widget w = NULL;
  assert(NULL != env);

  JAVARLOG1 (AWT_LOG, 2, "in sun_awt_motif_MComponentPeer_pHide(peer = %p)\n", peer);

  if (NULL == peer)
    {
      JAVARLOG0(AWT_LOG, 1, "peer is NULL in MComponentPeer.pHide!\n");
      return;
    }

  w = get_component_widget(env, peer);

  if (NULL == w)
    {
      JAVARLOG0(AWT_LOG, 1, "widget is NULL in MComponentPeer.pHide!\n");
      return;
    }

  LOCK_AWT();
  XtUnmanageChild(w);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pEnable(JNIEnv *env,
					  jobject peer)
{
  Widget w = get_component_widget(env, peer);

  if (NULL == w)
    {
      JAVARLOG0(AWT_LOG, 1, "widget is NULL in MComponentPeer.pEnable!\n");
      return;
    }

  LOCK_AWT();
  XtSetSensitive(w, True);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pDisable(JNIEnv *env,
					   jobject peer)
{
  Widget w = get_component_widget(env, peer);

  if (NULL == w)
    {
      JAVARLOG0(AWT_LOG, 1, "widget is NULL in MComponentPeer.pDisable!\n");
      return;
    }

  LOCK_AWT();
  XtSetSensitive(w, False);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pReshape(JNIEnv *env,
					   jobject peer,
					   jint x,
					   jint y,
					   jint width,
					   jint height)
{
  Widget widget = get_component_widget(env, peer);

  if (NULL == widget)
    {
      JAVARLOG0(AWT_LOG, 1, "widget is NULL in MComponentPeer.pReshape!\n");
      return;
    }

  LOCK_AWT();
  XtVaSetValues(widget,
		XmNx, x,
		XmNy, y,
		XmNwidth, width,
		XmNheight, height,
		NULL);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pAddRepaint(JNIEnv *env,
					      jobject peer,
					      jint x,
					      jint y,
					      jint width,
					      jint height)
{
  JAVARLOG5 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_pAddRepaint(peer = %p, x = %d, y = %d, width = %d, height = %d)\n",
	  peer, x, y, width, height);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pTriggerRepaint(JNIEnv *env,
						  jobject peer)
{
  JAVARLOG1 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_pTriggerRepaint(peer = %p)\n", peer);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pDispose(JNIEnv *env,
					   jobject peer)
{
  Widget w = get_component_widget(env, peer);

  JAVARLOG1 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_pDispose(peer = %p)\n", peer);

  if (NULL == w)
    {
      JAVARLOG0(AWT_LOG, 1, "widget is NULL in MComponentPeer.pDispose!\n");
      return;
    }

  LOCK_AWT();
  XtDestroyWidget(w);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pMakeCursorVisible(JNIEnv *env,
						     jobject peer)
{
  JAVARLOG1 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_pMakeCursorVisible(peer = %p)\n", peer);
}

JNIEXPORT jobject JNICALL
Java_sun_awt_motif_MComponentPeer_pGetLocationOnScreen(JNIEnv *env,
						       jobject peer)
{
  JAVARLOG1 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_pGetLocationOnScreen(peer = %p)\n", peer);
  return NULL;
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_setForeground(JNIEnv *env,
						jobject peer,
						jobject foreground)
{
  XColor fg;
  Widget w = get_component_widget(env, peer);
  jclass color = (*env)->GetObjectClass(env, foreground);
  jmethodID getRed = (*env)->GetMethodID(env, color, "getRed", "()I");
  jmethodID getGreen = (*env)->GetMethodID(env, color, "getGreen", "()I");
  jmethodID getBlue = (*env)->GetMethodID(env, color, "getBlue", "()I");
  jint red, green, blue;

  if (NULL == w) /* XXX Hm, how could this happend, and what should we do? */
    {
      JAVARLOG0(AWT_LOG,1,"widget is NULL in MComponentPeer.setForeground!\n");
      return;
    }

  red = (*env)->CallIntMethod(env, foreground, getRed);
  green = (*env)->CallIntMethod(env, foreground, getGreen);
  blue = (*env)->CallIntMethod(env, foreground, getBlue);

  JAVARLOG2 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_setForeground(peer = %p, foreground = %p)\n", peer, foreground);
  JAVARLOG3 (AWT_LOG, 2, "red = %d, green = %d, blue = %d\n", red, green, blue);

  fg.red = (red << 8) | red;
  fg.green = (green << 8) | red;
  fg.blue = (blue << 8) | red;
  fg.flags = DoRed | DoGreen | DoBlue;

  LOCK_AWT();
  XAllocColor(_awt_Display, DefaultColormapOfScreen(_awt_Screen),
	      &fg);
  XtVaSetValues(w,
		XmNforeground, fg.pixel,
		NULL);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_setBackground(JNIEnv *env,
						jobject peer,
						jobject background)
{
  XColor bg;
  Widget w = get_component_widget(env, peer);
  jclass color = (*env)->GetObjectClass(env, background);
  jmethodID getRed = (*env)->GetMethodID(env, color, "getRed", "()I");
  jmethodID getGreen = (*env)->GetMethodID(env, color, "getGreen", "()I");
  jmethodID getBlue = (*env)->GetMethodID(env, color, "getBlue", "()I");
  jint red, green, blue;

  if (NULL == w) /* XXX Hm, how could this happend, and what should we do? */
    {
      JAVARLOG0(AWT_LOG,1,"widget is NULL in MComponentPeer.setBackground!\n");
      return;
    }

  red = (*env)->CallIntMethod(env, background, getRed);
  green = (*env)->CallIntMethod(env, background, getGreen);
  blue = (*env)->CallIntMethod(env, background, getBlue);

  JAVARLOG2 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_setBackground(peer = %p, background = %p)\n", peer, background);
  JAVARLOG3 (AWT_LOG, 2, "red = %d, green = %d, blue = %d\n", red, green, blue);

  bg.red = (red << 8) | red;
  bg.green = (green << 8) | red;
  bg.blue = (blue << 8) | red;
  bg.flags = DoRed | DoGreen | DoBlue;

  LOCK_AWT();
  XAllocColor(_awt_Display, DefaultColormapOfScreen(_awt_Screen),
	      &bg);
  XmChangeColor(w, bg.pixel);
  XtVaSetValues(w,
		XmNbackground, bg.pixel,
		NULL);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_setFont(JNIEnv *env,
					  jobject peer,
					  jobject font)
{
  JAVARLOG2 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_setFont(peer = %p, font = %p)\n", peer, font);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_setCursor(JNIEnv *env,
					    jobject peer,
					    jobject cursor)
{
  Widget w = get_component_widget(env, peer);
  jclass cursor_class = (*env)->GetObjectClass(env, cursor);
  jfieldID type;
  jint cursor_type;

  JAVARLOG2 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_setCursor(peer = %p, cursor = %p)\n", peer, cursor);

  if (NULL == w)
    {
      JAVARLOG0(AWT_LOG,1,"widget is NULL in MComponentPeer.setCursor!\n");
      return;
    }

  type = (*env)->GetFieldID(env, cursor_class, "type", "I");
  cursor_type = (*env)->GetIntField(env, cursor, type);

  JAVARLOG1 (AWT_LOG, 2, "+  cursor_type = %d\n", cursor_type);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_handleEvent(JNIEnv *env,
					      jobject peer,
					      jobject event)
{
  jclass event_class;
  jmethodID getID;
  jint id;

  JAVARLOG2 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_handleEvent(peer = %p, event = %p)\n",
	     peer, event);

  event_class = (*env)->GetObjectClass(env, event);
  getID = (*env)->GetMethodID(env, event_class, "getID", "()I");
  id = (*env)->CallIntMethod(env, event, getID);

  JAVARLOG1 (AWT_LOG, 2, "+  event->id = %d\n", id);

  /* XXX what do I do here? */
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_requestFocus(JNIEnv *env,
					       jobject peer)
{
  JAVARLOG1 (AWT_LOG, 2, "In sun_awt_motif_MComponentPeer_requestFocus(peer = %p)\n", peer);
}
