/* -*- Mode: C; c-file-style: "gnu" -*-
   graphics.c -- native methods for sun/awt/motif/X11Graphics.
   Created: Chris Toshok <toshok@hungry.com>, 18-Sep-1998.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#include "config.h"
#include "jni.h"

#include "common.h"

#include <stdio.h>
#include <stdlib.h>

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_createFromComponent(JNIEnv *env,
						   jobject graphics,
						   jobject component_peer)
{
  GraphicsData *data;
  Widget widget = get_component_widget(env, component_peer);

  data = (GraphicsData*)malloc(sizeof(GraphicsData));
  if (data == NULL)
    {
      /* throwException.... XXX */
      return;
    }

  if (!XtIsRealized(widget))
    XtRealizeWidget(widget);

  data->widget = widget;
  data->gc = XCreateGC(_awt_Display,
		       XtWindow(data->widget),
		       0, NULL);

  set_graphics_data(env, graphics, data);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_createFromGraphics(JNIEnv *env,
						  jobject graphics,
						  jobject graphics_to_copy)
{
  GraphicsData* data = get_graphics_data(env, graphics_to_copy);
  GraphicsData* new_data;

  new_data = (GraphicsData*)malloc(sizeof(GraphicsData));
  if (data == NULL)
    {
      /* throwException.... XXX */
      return;
    }

  new_data->widget = data->widget;
  new_data->gc = XCreateGC(_awt_Display, XtWindow(new_data->widget), 0, NULL);

  if (new_data->gc == NULL)
    {
      /* hmm... XXX */
      return;
    }

  XCopyGC(_awt_Display, data->gc, ~0/*everything*/, new_data->gc);

  set_graphics_data(env, graphics, new_data);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_imageCreate(JNIEnv *env,
					   jobject obj,
					   jobject umm)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_pSetFont(JNIEnv *env,
					jobject obj,
					jobject font)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_pSetForeground(JNIEnv *env,
					      jobject obj,
					      jobject color)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_dispose(JNIEnv *env,
				       jobject graphics)
{
  GraphicsData* data = get_graphics_data(env, graphics);

  if (data != NULL)
    {
      XFreeGC(_awt_Display, data->gc);
      free(data);
      set_graphics_data(env, graphics, NULL);
    }
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_setPaintMode(JNIEnv *env,
					    jobject graphics)
{
  GraphicsData* data = get_graphics_data(env, graphics);

  XSetFunction(_awt_Display, data->gc, GXcopy);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_setXORMode(JNIEnv *env,
					  jobject graphics,
					  jobject umm /* XXX */)
{
  GraphicsData* data = get_graphics_data(env, graphics);

  XSetFunction(_awt_Display, data->gc, GXxor);
}

JNIEXPORT jobject JNICALL
Java_sun_awt_motif_X11Graphics_getClipBounds(JNIEnv *env,
					     jobject obj)
{
  return NULL;
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_changeClip(JNIEnv *env,
					  jobject obj,
					  jint x,
					  jint y,
					  jint width,
					  jint height,
					  jboolean umm)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_removeClip(JNIEnv *env,
					  jobject obj)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_clearRect(JNIEnv *env,
					 jobject graphics,
					 jint x,
					 jint y,
					 jint width,
					 jint height)
{
  GraphicsData *data = get_graphics_data(env, graphics);

  XClearArea(_awt_Display,
	     XtWindow(data->widget),
	     x, y, width, height, False);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_fillRect(JNIEnv *env,
					jobject graphics,
					jint x,
					jint y,
					jint width,
					jint height)
{
  GraphicsData *data = get_graphics_data(env, graphics);

  XFillRectangle(_awt_Display,
		 XtWindow(data->widget),
		 data->gc,
		 x, y, width, height);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_drawRect(JNIEnv *env,
					jobject graphics,
					jint x,
					jint y,
					jint width,
					jint height)
{
  GraphicsData *data = get_graphics_data(env, graphics);

  XDrawRectangle(_awt_Display,
		 XtWindow(data->widget),
		 data->gc,
		 x, y, width, height);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_drawSFChars(JNIEnv *env,
					   jobject obj,
					   jcharArray arr,
					   jint offset,
					   jint len,
					   jint width,
					   jint height)
{
}


JNIEXPORT jint JNICALL
Java_sun_awt_motif_X11Graphics_drawMFCharsSegment(JNIEnv *env,
						  jobject obj,
						  jboolean umm1,
						  jobject umm2,
						  jobject umm3,
						  jbyteArray umm4,
						  jint umm5,
						  jint umm6,
						  jint umm7)
{
  return 0;
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_drawBytes(JNIEnv *env,
					 jobject obj,
					 jbyteArray arr,
					 jint off,
					 jint len,
					 jint x,
					 jint y)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_drawLine(JNIEnv *env,
					jobject graphics,
					jint x1,
					jint y1,
					jint x2,
					jint y2)
{
  GraphicsData* data = get_graphics_data(env, graphics);

  XDrawLine(_awt_Display,
	    XtWindow(data->widget),
	    data->gc,
	    x1, y1,
	    x2, y2);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_copyArea(JNIEnv *env,
					jobject obj,
					jint x,
					jint y,
					jint width,
					jint height,
					jint dest_x,
					jint dest_y)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_drawRoundRect(JNIEnv *env,
					     jobject obj,
					     jint x,
					     jint y,
					     jint width,
					     jint height,
					     jint umm1,
					     jint umm2)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_fillRoundRect(JNIEnv *env,
					     jobject obj,
					     jint x,
					     jint y,
					     jint width,
					     jint height,
					     jint umm1,
					     jint umm2)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_drawPoly(JNIEnv *env,
					jobject obj,
					jintArray xs,
					jintArray ys,
					jint num_points,
					jboolean umm1)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_fillPolygon(JNIEnv *env,
					   jobject obj,
					   jintArray xs,
					   jintArray ys,
					   jint num_points)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_drawOval(JNIEnv *env,
					jobject obj,
					jint x,
					jint y,
					jint umm1,
					jint umm2)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_fillOval(JNIEnv *env,
					jobject obj,
					jint x,
					jint y,
					jint umm1,
					jint umm2)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_drawArc(JNIEnv *env,
				       jobject obj,
				       jint umm1,
				       jint umm2,
				       jint umm3,
				       jint umm4,
				       jint umm5,
				       jint umm6)
{
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_X11Graphics_fillArc(JNIEnv *env,
				       jobject obj,
				       jint umm1,
				       jint umm2,
				       jint umm3,
				       jint umm4,
				       jint umm5,
				       jint umm6)
{
}
